#include <assert.h>
#include "trace.h"

/* 
  This function is called to parse an instruction and determine which
  instruction type counters ought to be incremented.  These counters are used to
  calculate the instruction mixes reported when tracing is halted. You should
  only have to make modifications to this function in order to complete the lab
  assignment.

  Some counters are already implemented.  Those whose values are simply set to
  -1 are types that need to be detected and incremented by you. The custom
  counters are there for you to use only if you need them to justify your design
  in the final lab problem.

  Each trace entry passed to the function contains the instruction in byte
  array form (4 8-bit integers).In other words, the binary representation of
  each array entry is 8 bits of the instruction. Unfortunately SPARC v9
  opcodes and register ids do not all happen to be 8 bits long each, which
  means that you will have to do some work to parse the instruction and
  determine the value of these ISA fields. 
  
  You may find it more intuitive to mask out the needed fields with & and >>.
  Feel free to do so if it makes more sense to you.
*/


static void mixParser( base_trace_t *bt, trace_entry_t *ent) {
  // instr[0] is the highest order 8 bits, instr[3] is the lowest order bits
  uint8 * instr = ent->value.text; 

  // These instruction types need to be counted by you to complete Problem 2.5
  // Remember to remove the = -1 declaration if you want the number to be printed
  bt->ld_count = -1;
  bt->st_count = -1;
  bt->reg_reg_ldst_count = -1;
  bt->reg_reg_rs2_zero_ldst_count = -1;
  bt->zero_offset_ldst_count = -1;
  bt->nonzero_offset_ldst_count = -1;
	
  // These instruction counters may be used for your argument in Problem 3.2
  bt->custom_count1 = -1;
  bt->custom_count2 = -1;
  bt->custom_count3 = -1;

  // Refer to the SPARCv9 handout to see the opcode mappings
  if(instr[0] < 64) { 	                 // op[1:0] = 0
    if(instr[0] == 1 && instr[1] == 0 && 
       instr[2] == 0 && instr[3] == 0) { // op2[2:0] = 4, rd = 0, imm22 = 0
      bt->noop_count++;    // NOOP
    } else {
      bt->branch_count++;  // some kind of branch, jump or trap
    }       
  } else if(instr[0] < 128) {            // op[1:0] = 1
    bt->call_count++;      // call instruction
  } else if(instr[0] < 192) {            // op[1:0] = 2
    if(instr[0] % 2 == 0) {              // op3[5] = 0
      bt->arth_count++;    // arithmetic
    } else {                             // op3[5] = 1
      if(instr[1] < 64 ) {               // op3[4] = 0, op3[3:0] < 8 
	bt->arth_count++;  // arithmetic
      } else if(instr[1] >= 160 && 
		instr[1] <= 168){        // op3[4] = 1, op3[3:0] = 4 or 5
	bt->fpop_count++;  // some type of floating point operation
      } else {
	bt->misc_count++;  // miscellaneous
      }       
    }       
  } else {                               // op[1:0] = 3
    // Some kind of Load or Store
    // Your 2.3 code should go here
    bt->ldst_count++;
  }  
}
